"""Enum representing the possible tesselation types."""

from enum import Enum


class TessellationType(str, Enum):
    """Enum representing the possible tesselation types."""

    SQUARE_TRANSLATION = "SQUARE_TRANSLATION"
